#include <stdio.h>
#include <stdlib.h>
#include <time.h>

#define NDEALS 3000 	/* Numero distribuzioni */
#define NPLAYERS 6	/* Numero giocatori */

typedef enum {clubs, diamonds, hearts, spades} cdhs; // Segni delle carte: clubs=fiori, diamonds=quadri, hearts=cuori, spades=picche

// Struttura rappresentante una carta da gioco 
struct card {
	int pips; 	// numero
	cdhs suit; 	// segno
};

typedef struct card  card; // Definizione del nuovo tipo card a partire dalla struct card

// PROTOTIPI
card assign_values(int pips, cdhs suit);
void prn_card_values(card *c_ptr);
void play_poker(card deck[52]);
void shuffle(card deck[52]);
void swap(card *p, card *q);
void deal_the_cards(card deck[52], card hand[NPLAYERS][5]);
int is_flush(card h[5]);

// Main function
int main(void){
	cdhs suit; 	// Variabile segno della carta
	int i, pips; 	// Variabili i e numero carta
	card deck[52];	// deck=mazzo di 52 carte (array). Ogni carta e' una struct card

	for(i=0; i<52;++i){	// Per ogni carta del mazzo				
		pips= i%13 + 1;	// Numero va da 1 a 13 (13 carte per ogni seme)
		if(i<13)				// Prime 13 carte del mazzo sono inizializzate fiori
			suit=clubs;
		else if(i<26)
			suit=diamonds;			// Carte da 14 a 26 del mazzo sono inizializzate quadri
		else if(i<39)
			suit=hearts;			// Carte da 27 a 39 del mazzo sono inizializzate cuori
		else 
			suit=spades;			// Carte da 40 a 52 del mazzo sono inizializzate picche
		deck[i]=assign_values(pips,suit);	// Assegnazione della carta al mazzo (deck)	
	}
	for(i=26; i<39; ++i){
		prn_card_values(&deck[i]);	// Stampa i cuori (come esempio)
	}
	play_poker(deck);			// Gioca a pocker con il mazzo (deck) appena generato
	return 0;
}

// Asegna valori numero (pips) e segno (suit) ad una carta
card assign_values(int pips, cdhs suit){
	card c;
	c.pips=pips; 	// Inizializzazione membro pips della struct c (operatore .)
	c.suit=suit;	// Inizializzazione membro suit della struct c (operatore .)
	return c;
}

// Stampa valori numero (pips) e segno (suit) di una carta
void prn_card_values(card *c_ptr){
	int pips=c_ptr->pips; 	// Accesso tramite operatore -> al membro pips della carta puntata dal puntatore c_ptr
	cdhs suit=c_ptr->suit;	// Accesso tramite operatore -> al membro suit della carta puntata dal puntatore c_ptr
	char *suit_name;	// Stringa

	if(suit==clubs)			// Assegnazione stringa relativa al segno della carta
		suit_name="clubs"; 
	else if(suit==diamonds)
		suit_name="diamonds";
	else if(suit==hearts)
		suit_name="hearts";
	else if(suit==spades)
		suit_name="spades";
	printf("card: %2d of %s\n", pips, suit_name);	// Stampa di numero (pips) e segno (suit) della carta
}

// Calcola con una tecnica di campionamento la probabilita' di un colore (5 carte dello stesso seme in una mano)
void play_poker(card deck[52]){
	int flush_cnt=0, hand_cnt=0; // Contatori di mescolamento carte e numero di mani
	int i,j;
	card hand[NPLAYERS][5]; // Ogni giocatore riceve 5 carte (array bidimensionale)

	srand(time(NULL)); // Init generatore numeri casuali
	
	for(i=0; i<NDEALS;++i){ 	// Per ogni mano di gioco	
		shuffle(deck);		// Mescola il mazzo
		deal_the_cards(deck,hand);	// Da' le carte dal mazzo deck e popola la struttura dati hand
		for(j=0; j<NPLAYERS;++j){	// Per ciascun giocatore
			++hand_cnt;
			//for(int k=0; k<5;++k){
			//	prn_card_values(&deck[i]);
			//}
			if(is_flush(hand[j])){	// Controlla se ha in mano un "colore" (5 carte dello stesso seme)
				++flush_cnt;	// In caso positivo incrementa il contatore e aggiorna la probabilita'
				printf("%s%d\n%s%d\n%s%f\n\n",
				"	Hand number:  ", hand_cnt,  // NB: NDEALS*NPLAYERS=3000*6=18000 mani in totale
				"	Flush number:  ", flush_cnt,
				"Flush probability:  ",(double) flush_cnt / hand_cnt);
			}
		}		
	}
}

// Mescolamento delle carte del mazzo deck
void shuffle(card deck[52]){
	int i,j;
	for(i=0; i<52; ++i){
		j=rand() % 52;
		swap(&deck[i], &deck[j]);
	}
}

// Scambio di posizione di due carte nel mazzo  
void swap(card *p, card *q){
	card tmp;
	tmp=*p;
	*p=*q;
	*q=tmp;
}

// Da' le carte: cioe' popola la struttura dati hand (matrice bidimensionale con NPLAYERS righe e 5 colonne)
// Ogni riga contiene le 5 carte di ciascuno degli NPLAYERS giocatori
void deal_the_cards(card deck[52], card hand[NPLAYERS][5]){
	int card_cnt=0, i, j;
	for(j=0; j<5;++j){
		for(i=0; i<NPLAYERS;++i){
			hand[i][j]=deck[card_cnt++]; 	// Prende la prossima carta del mazzo e la da' ad un giocatore
							// NB: una carta per giocatore in modo ciclico, come nel poker		
		}
	}

}

// Controlla se le 5 carte di un giocatore sono un "colore" (5 carte dello stesso seme in una mano, detto "flush" in inglese)
int is_flush(card h[5]){
	int i;
	for(i=1; i<5;++i)
		if(h[i].suit != h[0].suit)
			return 0;
	return 1;
}


// gcc poker.c -o poker
// ./poker

